// Copyright 2017 The Johns Hopkins University Applied Physics Laboratory.
// Licensed under the MIT License. See LICENSE.txt in the project root for full license information.



#include <eopcc/grid_structs.hpp>

namespace eopcc {


void MaxHeightGrid(GridModel<double>& max_height_grid, const PointCloud& point_cloud, Point shift) {

  max_height_grid.setAll(max_height_grid.GetInvalidValue());

  for (size_t i=0; i < point_cloud.size(); ++i) {
    Point pt = point_cloud.loc[i] + shift;
    if (max_height_grid.is_inside(pt.x,pt.y)) {
      max_height_grid.at(pt.x, pt.y) = std::max(max_height_grid.at(pt.x, pt.y), pt.z);
    }
  }

}


GridModel<double> MaxHeightGrid(const PointCloud& point_cloud, const GridModel<double>::spacing_t& grid_spacing) {
  Extent extent = point_cloud.extent;
  extent.min_x -= std::get<0>(grid_spacing)/2.0;
  extent.max_x += std::get<0>(grid_spacing)/2.0;
  extent.min_y -= std::get<1>(grid_spacing)/2.0;
  extent.max_y += std::get<1>(grid_spacing)/2.0;

  GridModel<double> max_height_grid(extent, grid_spacing);
  max_height_grid.SetWKT(point_cloud.wkt_spatial_ref);


  for (size_t i=0; i < point_cloud.size(); ++i) {
    Point pt = point_cloud.loc[i];
    if (max_height_grid.is_inside(pt.x,pt.y)) {
      max_height_grid.at(pt.x, pt.y) = std::max(max_height_grid.at(pt.x, pt.y), pt.z);
    }
  }

  return max_height_grid;
}

GridModel<double> AverageHeightGrid(const PointCloud& point_cloud, const GridModel<double>::spacing_t& grid_spacing) {
  Extent extent = point_cloud.extent;
  GridModel<double> sum_height_grid(extent, grid_spacing);
  GridModel<int> count_grid(extent, grid_spacing);
  sum_height_grid.setAll(0.0);
  count_grid.setAll(0);

  for (size_t i=0; i < point_cloud.size(); ++i) {
    Point pt = point_cloud.loc[i];

    sum_height_grid.at(pt.x, pt.y) += pt.z;
    count_grid.at(pt.x, pt.y) += 1;
  }


  for (size_t i=0; i < sum_height_grid.grid.size(); ++i) {
    if (count_grid.grid[i] == 0) {
      sum_height_grid.grid[i] = -std::numeric_limits<double>::max();
    } else {
      sum_height_grid.grid[i] = sum_height_grid.grid[i] / count_grid.grid[i];
    }
  }

  return sum_height_grid;
}


} // namespace eopcc
